package tools.descartes.dql.connector.kieker.filter;

import kieker.analysis.IProjectContext;
import kieker.analysis.plugin.annotation.InputPort;
import kieker.analysis.plugin.annotation.OutputPort;
import kieker.analysis.plugin.annotation.Plugin;
import kieker.analysis.plugin.filter.AbstractFilterPlugin;
import kieker.common.configuration.Configuration;
import kieker.common.record.IMonitoringRecord;
import kieker.common.record.controlflow.OperationExecutionRecord;
import tools.descartes.dql.connector.kieker.structures.classes.ClassMap;
import tools.descartes.dql.connector.kieker.structures.classes.ClassMapKey;
import tools.descartes.dql.connector.kieker.structures.classes.ClassMapValue;
import tools.descartes.dql.connector.kieker.structures.interfaces.InterfaceMap;
import tools.descartes.dql.connector.kieker.structures.interfaces.InterfaceMapKey;
import tools.descartes.dql.connector.kieker.structures.interfaces.InterfaceMapValue;



@Plugin(
		name = "Class and interface filter, CAIF",
		description = "Extracts class-information from incoming monitoring records",
		outputPorts = {
				@OutputPort(name = ClassAndInterfaceFilter.OUTPUT_PORT_NAME_MATCHES,
						description = "forwardingOpExRecord",
						eventTypes = { IMonitoringRecord.class }),
				@OutputPort(name = ClassAndInterfaceFilter.OUTPUT_PORT_NAME_RAW,
						description = "Outputs untouched information",
						eventTypes = { IMonitoringRecord.class })}
		)
public class ClassAndInterfaceFilter extends AbstractFilterPlugin {

	public static final String INPUT_PORT_NAME = "newMonitoringRecord";
	public static final String OUTPUT_PORT_NAME_MATCHES = "forwardingOpExRecord";
	public static final String OUTPUT_PORT_NAME_RAW = "untouchedIMonitoringRecord";

	public ClassAndInterfaceFilter(final Configuration configuration, final IProjectContext projectContext) {
		super(configuration, projectContext);
	}



	@InputPort(
			name = ClassAndInterfaceFilter.INPUT_PORT_NAME,
			description = "Extract class-information from monitoring record",
			eventTypes = { IMonitoringRecord.class })
	public void runFilter(final Object record) {
		if (record instanceof OperationExecutionRecord) {
			final OperationExecutionRecord opExRecord = (OperationExecutionRecord) record;

			final long timestamp = opExRecord.getLoggingTimestamp();
			//LoggingTimestampConverter.convertLoggingTimestampToUTCString(timestamp);

			final String hostName = opExRecord.getHostname();
			final String signature = opExRecord.getOperationSignature();
			
			final long exTime = opExRecord.getTout()-opExRecord.getTin();
			final String classPattern = "[\\p{Graph}]+((\\.)(\\w)*[<(].*[>)])";//String-pattern to extract the component information
			final String interfacePattern = "((\\.)(\\w)*[<(].*[>)])";//String-pattern to extract the component information
			final String requestID = opExRecord.getSessionId();

			//			final long traceID = opExRecord.getTraceId();
			//			final long eoi = opExRecord.getEoi();
			//			final long ess = opExRecord.getEss();


			String className = null;
			String interfaceName = null;

			try {
				String[] redundantInfo=signature.split(classPattern);
				
				//(redundantInfo + classAndInterfaceString) - redundantInfo = classAndInterfaceString
				String classAndInterfaceString = signature;
				if (redundantInfo.length > 0){
					classAndInterfaceString = signature.substring(redundantInfo[0].length());
				}
				
				//extraction of the class information
				String[] componentSeparatedByNonWordChar = classAndInterfaceString.split(interfacePattern);
				className = componentSeparatedByNonWordChar[0].trim();

				//extraction of the interface information
				interfaceName = classAndInterfaceString.substring(className.length()+1).trim();				

			} catch (Exception e) {
				e.printStackTrace();
				/*
				 * TODO
				 */
			}

			long classID = putIntoClassMap(timestamp,hostName, className, requestID);
			putIntoInterfaceMap(timestamp,interfaceName, classID, exTime, requestID);
			super.deliver(OUTPUT_PORT_NAME_MATCHES, opExRecord);
		} else {
			super.deliver(OUTPUT_PORT_NAME_RAW, record);
		}
	}

	public static synchronized long putIntoClassMap(long timestamp,String hostName, String className, String requestID){
		ClassMapKey classKey = new ClassMapKey(hostName,className, requestID);
		ClassMap.getInstance().put(classKey, new ClassMapValue(timestamp)); 
		return  ClassMap.getInstance().get(classKey).getID();
	}

	public static synchronized void putIntoInterfaceMap(long timestamp, String interfaceName, long classID, long exTime, String requestID){
		InterfaceMapKey interfaceKey = new InterfaceMapKey(interfaceName, classID,requestID);
		InterfaceMap.getInstance().put(interfaceKey,new InterfaceMapValue(timestamp,exTime)); 
	}

	@Override
	public Configuration getCurrentConfiguration() {
		return new Configuration();
	}

}
